/*!
 * jQuery hTmpl Plugin - inspired by Remarkable Pre-processing Linker
 * version: v1.5-2019.08.02
 * @requires jQuery v1.0 or later
 * Copyright (c) 2019 Kang,dong-hyeon
 * Examples and documentation at: http://hidongh.tistory.com/11
 * Dual licensed under the MIT and GPL licenses.
 */
var g_hTmplRemarkBlock = {};

(function($) {
"use strict";

/**
 * 리마커블 결합기 개념을 용용하여 javascript로 구현한 html templating 함수
 * (관련 논문: 웹페이지 코드분리를 위한 리마커블 전처리기의 설계 및 구현. 강동현. 2007.8)
 *
 * @param obj : a key-value object for mapping to templating area
 * @return htmlString for rendering
 */
$.fn.hTmpl = function(obj) {
	var tmplKey = $(this).selector; // tmpl 영역 식별자
	var tmpl = g_hTmplRemarkBlock[tmplKey]; // tmpl 캐시 참조
	if (!tmpl) { // 캐시에 없으면
		tmpl = $(this).html(); // tmpl 영역
		if (!tmpl) return "";
		g_hTmplRemarkBlock[tmplKey] = tmpl; // tmpl 캐싱
	}
	
	var aTag = "{{";
	var oTag = "}}";
	var idx = tmpl.lastIndexOf(aTag);
	while (idx > -1) {
		var idxEnd = tmpl.indexOf(oTag, idx);
		var key = $.trim(tmpl.substring(idx + aTag.length, idxEnd));
		
		var arr = ["=", "if/", "ifNot/", "/if", "/EndifForInterProcessing/"];
		var i = 0;
		for (; i < arr.length; i++) {
			if (key.indexOf(arr[i]) == 0) {
				key = $.trim(key.substring(arr[i].length));
				break;
			}
		}
		
		var subObj = obj;
		var idxDot = key.indexOf(".");
		while (idxDot > -1) {
			var subKey = key.substring(0, idxDot);
			subObj = subObj[subKey];
			key = key.substring(idxDot + 1);
			idxDot = key.indexOf(".");
		}
		var item = subObj[key];
		
		if (i == 0) {
			item += "";
			item = item.split("&").join("&amp;").split("<").join("&lt;").split(">").join("&gt;").split('"').join("&#34;").split("'").join("&#39;");
		} else if (i == 1 || i == 2) {
			if (i == 1 && !item || i == 2 && item) {
				idxEnd = tmpl.indexOf(arr[4], idxEnd + 1);
				idxEnd = tmpl.indexOf(oTag, idxEnd + 1);
			} else {
				var idxEndif = tmpl.indexOf(arr[4], idxEnd + 1);
				tmpl = tmpl.substring(0, idxEndif) + aTag + arr[4] + oTag + tmpl.substring(idxEndif + arr[4].length + oTag.length);
			}
			item = "";
		} else if (i == 3) {
			item = arr[4] + oTag;
		} else if (i == 4) {
			item = "";
		} else {
			item += "";
		}
		
		tmpl = tmpl.substring(0, idx) + item + tmpl.substring(idxEnd + oTag.length);
		idx = tmpl.lastIndexOf(aTag);
	}
	return tmpl;
};

})(jQuery);